Method: ChefConfig::Config.embedded_dir

Defined in:
lib/chef-config/config.rb

.embedded_dirObject

If installed via an omnibus installer, this gives the path to the “embedded” directory which contains all of the software packaged with omnibus. This is used to locate the cacert.pem file on windows.



726
727
728
729
730
731
732
733
734
# File 'lib/chef-config/config.rb', line 726

def self.embedded_dir
  Pathname.new(_this_file).ascend do |path|
    if path.basename.to_s == "embedded"
      return path.to_s
    end
  end

  nil
end