Method: ChefConfig::Config.env
- Defined in:
- lib/chef-config/config.rb
.env ⇒ Object
This provides a hook which rspec can stub so that we can avoid twiddling global state in tests.
575 576 577 |
# File 'lib/chef-config/config.rb', line 575 def self.env ENV end |